@@ -15,6 +15,7 @@ import subprocess
 import sys
 import tempfile
 import threading
+import argparse
 
 import gi
 gi.require_version('Gtk', '3.0')
@@ -30,6 +31,13 @@ import mintcommon.aptdaemon
 
 BUTTON_LABEL_MAX_LENGTH = 30
 
+# Used when launched by synaptic (via software-properties-gtk).
+# The return code tells synaptic to refresh the cache if sources have changed
+# If we try to refresh ourselves in this scenario, apt.Cache gets stuck waiting
+# for synaptic to exit...
+disable_refresh = False
+sources_changed = False
+
 FLAG_PATH = "/usr/share/iso-flag-png/%s.png"
 FLAG_SIZE = 16
 
@@ -731,7 +739,7 @@ class MirrorSelectionDialog(object):
 
         # Try to find out where we're located...
         try:
-            lookup = requests.get('http://geoip.ubuntu.com/lookup').text
+            lookup = requests.get('https://geoip.ubuntu.com/lookup').text
             cur_country_code = re.search('<CountryCode>(.*)</CountryCode>', lookup).group(1)
             if cur_country_code == 'None': cur_country_code = None
         except Exception as detail:
@@ -829,6 +837,16 @@ class MirrorSelectionDialog(object):
 
 class Application(object):
     def __init__(self):
+        parser = argparse.ArgumentParser(description="Software sources for Linux Mint")
+        parser.add_argument("-n", "--no-update", action="store_true", help="Disable cache refresh prompting")
+        args = parser.parse_known_args()
+
+        try:
+            known_args = args[0]
+            global disable_refresh
+            disable_refresh = known_args.no_update
+        except (AttributeError, IndexError) as e:
+            print(e)
 
         # Prevent settings from being saved until the interface is fully loaded
         self._interface_loaded = False
@@ -917,6 +935,7 @@ class Application(object):
         self._ppa_treeview.set_headers_clickable(True)
         self._ppa_treeview.connect("row-activated", self.on_ppa_treeview_doubleclick)
         selection = self._ppa_treeview.get_selection()
+        selection.set_mode(Gtk.SelectionMode.MULTIPLE)
         selection.connect("changed", self.ppa_selected)
 
         self._ppa_model.set_sort_column_id(2, Gtk.SortType.ASCENDING)
@@ -941,6 +960,7 @@ class Application(object):
         self._repository_treeview.set_model(self._repository_model)
         self._repository_treeview.set_headers_clickable(True)
         repo_selection = self._repository_treeview.get_selection()
+        repo_selection.set_mode(Gtk.SelectionMode.MULTIPLE)
         repo_selection.connect("changed", self.repo_selected)
 
         self._repository_model.set_sort_column_id(2, Gtk.SortType.ASCENDING)
@@ -964,6 +984,7 @@ class Application(object):
         self._keys_treeview.set_model(self._keys_model)
         self._keys_treeview.set_headers_clickable(True)
         keys_selection = self._keys_treeview.get_selection()
+        keys_selection.set_mode(Gtk.SelectionMode.MULTIPLE)
         keys_selection.connect("changed", self.key_selected)
 
         self._keys_model.set_sort_column_id(1, Gtk.SortType.ASCENDING)
@@ -1356,18 +1377,22 @@ class Application(object):
         self.load_keys()
 
     def remove_key(self, widget):
-        selection = self._keys_treeview.get_selection()
-        (model, iter) = selection.get_selected()
-        if (iter != None):
-            key = model.get(iter, 0)[0]
-            image = Gtk.Image()
-            image.set_from_icon_name("dialog-password-symbolic", Gtk.IconSize.DIALOG)
-            if (self.show_confirmation_dialog(self._main_window, _("Are you sure you want to permanently remove this key?"), image, yes_no=True)):
+        image = Gtk.Image()
+        image.set_from_icon_name("dialog-password-symbolic", Gtk.IconSize.DIALOG)
+        if (self.show_confirmation_dialog(self._main_window, _("Are you sure you want to permanently remove the selected keys?"), image, yes_no=True)):
+            selection = self._keys_treeview.get_selection()
+            (model, indexes) = selection.get_selected_rows()
+            iters = []
+            for index in indexes:
+                iters.append(model.get_iter(index))
+            for iter in iters:
+                key = model.get(iter, 0)[0]
                 key.delete()
-                self.load_keys()
+            self.load_keys()
 
     def key_selected(self, selection):
-        self.builder.get_object("button_keys_remove").set_sensitive(True)
+        selection_count = selection.count_selected_rows()
+        self.builder.get_object("button_keys_remove").set_sensitive(selection_count >= 1)
 
     def add_ppa(self, widget):
         image = Gtk.Image()
@@ -1442,38 +1467,43 @@ class Application(object):
 
     def edit_ppa(self, widget):
         selection = self._ppa_treeview.get_selection()
-        (model, iter) = selection.get_selected()
-        if (iter != None):
-            repository = model.get(iter, 0)[0]
-            url = self.show_entry_dialog(self._main_window, _("Edit the URL of the PPA"), repository.line)
-            if url is not None:
-                repository.edit(url)
-                model.set_value(iter, 2, repository.get_ppa_name())
+        (model, indexes) = selection.get_selected_rows()
+        iter = model.get_iter(indexes[0])
+        repository = model.get(iter, 0)[0]
+        url = self.show_entry_dialog(self._main_window, _("Edit the URL of the PPA"), repository.line)
+        if url is not None:
+            repository.edit(url)
+            model.set_value(iter, 2, repository.get_ppa_name())
 
     def remove_ppa(self, widget):
-        selection = self._ppa_treeview.get_selection()
-        (model, iter) = selection.get_selected()
-        if (iter != None):
-            repository = model.get(iter, 0)[0]
-            if (self.show_confirmation_dialog(self._main_window, _("Are you sure you want to permanently remove this PPA?"), yes_no=True)):
+        if (self.show_confirmation_dialog(self._main_window, _("Are you sure you want to permanently remove the selected PPAs?"), yes_no=True)):
+            selection = self._ppa_treeview.get_selection()
+            (model, indexes) = selection.get_selected_rows()
+            iters = []
+            for index in indexes:
+                iters.append(model.get_iter(index))
+            for iter in iters:
+                repository = model.get(iter, 0)[0]
                 model.remove(iter)
                 repository.delete()
                 self.ppas.remove(repository)
 
     def ppa_selected(self, selection):
-        self.builder.get_object("button_ppa_edit").set_sensitive(True)
-        self.builder.get_object("button_ppa_remove").set_sensitive(True)
+        selection_count = selection.count_selected_rows()
+        self.builder.get_object("button_ppa_edit").set_sensitive(selection_count == 1)
+        self.builder.get_object("button_ppa_remove").set_sensitive(selection_count >= 1)
 
-        try:
-            self.builder.get_object("button_ppa_examine").set_sensitive(False)
-            (model, iter) = selection.get_selected()
-            if (iter != None):
+        self.builder.get_object("button_ppa_examine").set_sensitive(False)
+        if (selection_count == 1):
+            try:
+                (model, indexes) = selection.get_selected_rows()
+                iter = model.get_iter(indexes[0])
                 repository = model.get_value(iter, 0)
                 ppa_name = model.get_value(iter, 2)
                 if repository.selected and repository.line.startswith("deb http://ppa.launchpad.net"):
                     self.builder.get_object("button_ppa_examine").set_sensitive(True)
-        except Exception as detail:
-            print (detail)
+            except Exception as detail:
+                print (detail)
 
     def on_ppa_treeview_doubleclick(self, treeview, path, column):
         self.examine_ppa(None)
@@ -1481,28 +1511,29 @@ class Application(object):
     def examine_ppa(self, widget):
         try:
             selection = self._ppa_treeview.get_selection()
-            (model, iter) = selection.get_selected()
-            if (iter != None):
-                repository = model.get_value(iter, 0)
-                ppa_name = model.get_value(iter, 2)
-                if repository.selected and repository.line.startswith("deb http://ppa.launchpad.net"):
-                    line = repository.line.split()[1].replace("http://ppa.launchpad.net/", "")
-                    if line.endswith("/ubuntu"):
-                        line = line[:-7]
-                        ppa_owner, ppa_name = line.split("/")
-                        architecture = subprocess.getoutput("dpkg --print-architecture")
-                        ppa_file = "/var/lib/apt/lists/ppa.launchpad.net_%s_%s_ubuntu_dists_%s_main_binary-%s_Packages" % (ppa_owner, ppa_name, self.config["general"]["base_codename"], architecture)
-                        if os.path.exists(ppa_file):
-                            os.system("/usr/lib/linuxmint/mintSources/ppa_browser.py %s %s %s &" % (self.config["general"]["base_codename"], ppa_owner, ppa_name))
-                        else:
-                            print ("%s not found!" % ppa_file)
-                            self.show_error_dialog(self._main_window, _("The content of this PPA is not available. Please refresh the cache and try again."))
+            (model, indexes) = selection.get_selected_rows()
+            iter = model.get_iter(indexes[0])
+            repository = model.get_value(iter, 0)
+            ppa_name = model.get_value(iter, 2)
+            if repository.selected and repository.line.startswith("deb http://ppa.launchpad.net"):
+                line = repository.line.split()[1].replace("http://ppa.launchpad.net/", "")
+                if line.endswith("/ubuntu"):
+                    line = line[:-7]
+                    ppa_owner, ppa_name = line.split("/")
+                    architecture = subprocess.getoutput("dpkg --print-architecture")
+                    ppa_file = "/var/lib/apt/lists/ppa.launchpad.net_%s_%s_ubuntu_dists_%s_main_binary-%s_Packages" % (ppa_owner, ppa_name, self.config["general"]["base_codename"], architecture)
+                    if os.path.exists(ppa_file):
+                        os.system("/usr/lib/linuxmint/mintSources/ppa_browser.py %s %s %s &" % (self.config["general"]["base_codename"], ppa_owner, ppa_name))
+                    else:
+                        print ("%s not found!" % ppa_file)
+                        self.show_error_dialog(self._main_window, _("The content of this PPA is not available. Please refresh the cache and try again."))
         except Exception as detail:
             print (detail)
 
     def repo_selected(self, selection):
-        self.builder.get_object("button_repository_edit").set_sensitive(True)
-        self.builder.get_object("button_repository_remove").set_sensitive(True)
+        selection_count = selection.count_selected_rows()
+        self.builder.get_object("button_repository_edit").set_sensitive(selection_count == 1)
+        self.builder.get_object("button_repository_remove").set_sensitive(selection_count >= 1)
 
     def add_repository(self, widget):
         image = Gtk.Image()
@@ -1536,20 +1567,23 @@ class Application(object):
 
     def edit_repository(self, widget):
         selection = self._repository_treeview.get_selection()
-        (model, iter) = selection.get_selected()
-        if (iter != None):
-            repository = model.get(iter, 0)[0]
-            url = self.show_entry_dialog(self._main_window, _("Edit the URL of the repository"), repository.line)
-            if url is not None:
-                repository.edit(url)
-                model.set_value(iter, 2, repository.get_repository_name())
+        (model, indexes) = selection.get_selected_rows()
+        iter = model.get_iter(indexes[0])
+        repository = model.get(iter, 0)[0]
+        url = self.show_entry_dialog(self._main_window, _("Edit the URL of the repository"), repository.line)
+        if url is not None:
+            repository.edit(url)
+            model.set_value(iter, 2, repository.get_repository_name())
 
     def remove_repository(self, widget):
-        selection = self._repository_treeview.get_selection()
-        (model, iter) = selection.get_selected()
-        if (iter != None):
-            repository = model.get(iter, 0)[0]
-            if (self.show_confirmation_dialog(self._main_window, _("Are you sure you want to permanently remove this repository?"), yes_no=True)):
+        if (self.show_confirmation_dialog(self._main_window, _("Are you sure you want to permanently remove the selected repositories?"), yes_no=True)):
+            selection = self._repository_treeview.get_selection()
+            (model, indexes) = selection.get_selected_rows()
+            iters = []
+            for index in indexes:
+                iters.append(model.get_iter(index))
+            for iter in iters:
+                repository = model.get(iter, 0)[0]
                 model.remove(iter)
                 repository.delete()
                 self.repositories.remove(repository)
@@ -1721,6 +1755,11 @@ class Application(object):
 
 
     def enable_reload_button(self):
+        if disable_refresh:
+            global sources_changed
+            sources_changed = True
+            return
+
         if not self._info_revealer.get_reveal_child():
             if self._info_box.get_children() == []:
                 infobar = Gtk.InfoBar()
@@ -1931,3 +1970,5 @@ if __name__ == "__main__":
             add_repository_via_cli(ppa_line, codename, "-y" in args, use_ppas)
     else:
         Application().run()
+
+    exit(1 if sources_changed else 0)
